"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const event_stream_1 = __importDefault(require("event-stream"));
const moment_1 = __importDefault(require("moment"));
const mongodb_1 = require("mongodb");
const odata_v4_mongodb_1 = require("odata-v4-mongodb");
const odata_v4_server_1 = require("odata-v4-server");
const trader_service_1 = require("trader-service");
const connect_1 = __importDefault(require("../connect"));
const BufferItem_1 = require("../models/BufferItem");
const Candle_1 = require("../models/Candle");
const Chart_1 = require("../models/Chart");
const Indicator_1 = require("../models/Indicator");
const Series_1 = require("../models/Series");
const SeriesItem_1 = require("../models/SeriesItem");
const Session_1 = require("../models/Session");
const View_1 = require("../models/View");
const collectionName = "session";
let SessionController = class SessionController extends odata_v4_server_1.ODataController {
    get(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const db = yield connect_1.default();
            const mongodbQuery = odata_v4_mongodb_1.createQuery(query);
            if (mongodbQuery.query._id) {
                mongodbQuery.query._id = new mongodb_1.ObjectID(mongodbQuery.query._id);
            }
            const result = typeof mongodbQuery.limit === "number" && mongodbQuery.limit === 0
                ? []
                : yield db
                    .collection(collectionName)
                    .find(mongodbQuery.query)
                    .project(mongodbQuery.projection)
                    .skip(mongodbQuery.skip || 0)
                    .limit(mongodbQuery.limit || 0)
                    .sort(mongodbQuery.sort)
                    .map(e => new Session_1.Session(Object.assign(e, {
                    status: Session_1.Session.streams[e._id] ? "active" : "inactive"
                })))
                    .toArray();
            if (mongodbQuery.inlinecount) {
                result.inlinecount = yield db
                    .collection(collectionName)
                    .find(mongodbQuery.query)
                    .project(mongodbQuery.projection)
                    .count(false);
            }
            return result;
        });
    }
    getById(key, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const { projection } = odata_v4_mongodb_1.createQuery(query);
            // tslint:disable-next-line: variable-name
            const _id = new mongodb_1.ObjectID(key);
            const db = yield connect_1.default();
            const session = new Session_1.Session(yield db.collection(collectionName).findOne({ _id }, { projection }));
            session.status = Session_1.Session.streams[session._id.toHexString()]
                ? "active"
                : "inactive";
            return session;
        });
    }
    post(body) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!body.begin) {
                body.begin = moment_1.default()
                    .utc()
                    .toISOString();
            }
            const session = new Session_1.Session(body);
            const db = yield connect_1.default();
            const collection = yield db.collection(collectionName);
            session._id = (yield collection.insertOne(session)).insertedId;
            const { _id, type, asset, currency, exchange, period, begin, end, indicators, code, initialBalance } = session;
            if (type === "backtest") {
                const rs = trader_service_1.streamTradesBacktest({
                    exchange,
                    currency,
                    asset,
                    period,
                    start: begin,
                    end,
                    indicators: JSON.parse(indicators),
                    code,
                    initialBalance
                });
                let finalBalance;
                const tradesCollection = yield db.collection("trade");
                rs.pipe(event_stream_1.default.map((chunk, next) => {
                    const doc = JSON.parse(chunk);
                    doc.parentId = _id;
                    finalBalance = doc.amount;
                    tradesCollection.insertOne(doc, next);
                }));
                yield new Promise(resolve => {
                    rs.on("end", resolve);
                });
                finalBalance = Math.abs(finalBalance);
                yield collection.updateOne({ _id }, {
                    $set: {
                        finalBalance,
                        profit: finalBalance - initialBalance
                    }
                });
            }
            else if (type === "paper") {
                const stream = trader_service_1.streamTradesPaper({
                    exchange,
                    currency,
                    asset,
                    period: "" + period,
                    start: moment_1.default()
                        .utc()
                        .toISOString(),
                    indicators: JSON.parse(indicators),
                    code,
                    initialBalance
                });
                stream.pipe(event_stream_1.default.map((chunk, next) => __awaiter(this, void 0, void 0, function* () {
                    const doc = JSON.parse(chunk);
                    doc.parentId = _id;
                    const tradesCollection = yield db.collection("trade"); // UNDONE можно ли подсократить?
                    yield tradesCollection.insertOne(doc, next);
                    yield collection.updateOne({ _id }, {
                        $set: {
                            finalBalance: Math.abs(doc.amount),
                            profit: Math.abs(doc.amount) - initialBalance
                        }
                    });
                })));
                Session_1.Session.streams[_id.toHexString()] = stream;
            }
            return session;
        });
    }
    remove(key) {
        return __awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line: variable-name
            const _id = new mongodb_1.ObjectID(key);
            return (yield connect_1.default())
                .collection(collectionName)
                .deleteOne({ _id })
                .then(result => result.deletedCount);
        });
    }
    getTrades(result, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const db = yield connect_1.default();
            const collection = db.collection("trade");
            const mongodbQuery = odata_v4_mongodb_1.createQuery(query);
            const parentId = new mongodb_1.ObjectID(result._id);
            const trades = typeof mongodbQuery.limit === "number" && mongodbQuery.limit === 0
                ? []
                : yield collection
                    .find({ $and: [{ parentId }, mongodbQuery.query] })
                    .project(mongodbQuery.projection)
                    .skip(mongodbQuery.skip || 0)
                    .limit(mongodbQuery.limit || 0)
                    .sort(mongodbQuery.sort)
                    .toArray();
            if (mongodbQuery.inlinecount) {
                trades.inlinecount = yield collection
                    .find({ $and: [{ parentId }, mongodbQuery.query] })
                    .project(mongodbQuery.projection)
                    .count(false);
            }
            return trades;
        });
    }
    getBuffer(result) {
        return __awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line: variable-name
            const _id = new mongodb_1.ObjectID(result._id);
            const db = yield connect_1.default();
            const { exchange, currency, asset, period, begin, end, indicators } = yield db.collection(collectionName).findOne({ _id });
            const bufferItems = [];
            const parsedIndicators = JSON.parse(indicators);
            const rs = trader_service_1.streamBuffer({
                exchange,
                currency,
                asset,
                period,
                start: begin,
                end: end ||
                    moment_1.default()
                        .utc()
                        .toISOString(),
                indicators: parsedIndicators
            });
            rs.pipe(event_stream_1.default.map((chunk, next) => {
                const buffer = JSON.parse(chunk);
                // Candles
                const { time, open, high, low, close, volume } = buffer.candle;
                bufferItems.push(new BufferItem_1.BufferItem({
                    Candle: new Candle_1.Candle({
                        time,
                        open,
                        high,
                        low,
                        close,
                        volume
                    }),
                    Indicators: buffer.indicators.map(values => new Indicator_1.Indicator({
                        values
                    }))
                }));
                next();
            }));
            return new Promise(resolve => {
                rs.on("end", () => resolve(bufferItems));
            });
        });
    }
    getView(result) {
        return __awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line: variable-name
            const _id = new mongodb_1.ObjectID(result._id);
            const db = yield connect_1.default();
            const { exchange, currency, asset, period, begin, end, indicators } = yield db.collection(collectionName).findOne({ _id });
            const candles = [];
            const indicatorCharts = [];
            const parsedIndicators = JSON.parse(indicators);
            const rs = trader_service_1.streamBuffer({
                exchange,
                currency,
                asset,
                period,
                start: begin,
                end: end ||
                    moment_1.default()
                        .utc()
                        .toISOString(),
                indicators: parsedIndicators
            });
            rs.pipe(event_stream_1.default.map((chunk, next) => {
                const buffer = JSON.parse(chunk);
                // Candles
                const { time, open, high, low, close, volume } = buffer.candle;
                candles.push(new Candle_1.Candle({
                    time,
                    open,
                    high,
                    low,
                    close,
                    volume
                }));
                // Indicators
                buffer.indicators.forEach((value0, index0) => {
                    // добавить недостающие диаграммы
                    if (indicatorCharts.length === index0) {
                        indicatorCharts.push(new Chart_1.Chart({
                            Series: []
                        }));
                    }
                    value0.forEach((value1, index1) => {
                        // добавить недостающие серии
                        const series = indicatorCharts[index0].Series;
                        if (series.length === index1) {
                            series.push(new Series_1.Series({
                                Items: []
                            }));
                        }
                        // добавить индикаторы
                        series[index1].Items.push(new SeriesItem_1.SeriesItem({
                            time,
                            value: value1
                        }));
                    });
                });
                next();
            }));
            return new Promise(resolve => {
                rs.on("end", () => {
                    resolve(new View_1.View({
                        Candles: candles,
                        Indicators: indicatorCharts
                    }));
                });
            });
        });
    }
};
__decorate([
    odata_v4_server_1.odata.GET,
    __param(0, odata_v4_server_1.odata.query)
], SessionController.prototype, "get", null);
__decorate([
    odata_v4_server_1.odata.GET,
    __param(0, odata_v4_server_1.odata.key),
    __param(1, odata_v4_server_1.odata.query)
], SessionController.prototype, "getById", null);
__decorate([
    odata_v4_server_1.odata.POST,
    __param(0, odata_v4_server_1.odata.body)
], SessionController.prototype, "post", null);
__decorate([
    odata_v4_server_1.odata.DELETE,
    __param(0, odata_v4_server_1.odata.key)
], SessionController.prototype, "remove", null);
__decorate([
    odata_v4_server_1.odata.GET("Trades"),
    __param(0, odata_v4_server_1.odata.result),
    __param(1, odata_v4_server_1.odata.query)
], SessionController.prototype, "getTrades", null);
__decorate([
    odata_v4_server_1.odata.GET("Buffer"),
    __param(0, odata_v4_server_1.odata.result)
], SessionController.prototype, "getBuffer", null);
__decorate([
    odata_v4_server_1.odata.GET("View"),
    __param(0, odata_v4_server_1.odata.result)
], SessionController.prototype, "getView", null);
SessionController = __decorate([
    odata_v4_server_1.odata.type(Session_1.Session),
    odata_v4_server_1.Edm.EntitySet("Session")
], SessionController);
exports.SessionController = SessionController;
//# sourceMappingURL=Session.js.map